/*
** ipaddr.h - deals with the ip address tables
**
** Keith Zaback (krz@wcug.wwu.edu) Sep 98
*/

#ifndef INC_IPADDR_H
#define INC_IPADDR_H

/* includes */
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>

#include "buffer.h"
#include "furytypes.h"
#include "hdrcache.h"
#include "hash.h"
#include "fury.h"

#define FURY_IOPT_NONE		0x0000		/* no options */
#define FURY_IOPT_ZLIB		0x0001		/* they have zlib */

#define FURY_HOSTNAME_MAX	64

/* the entry */
typedef struct stIPADDR_ENTRY {
	char			hostname[FURY_HOSTNAME_MAX];
	struct in_addr		addr;		/* the actual address */
	int			port;		/* the port they are on */
	struct timeval		last_sent_time;	/* time we last sent something here */
	struct timeval		last_recv_time;	/* time we last rcvd something here */
	struct timeval		last_keepalive;
	int			keepalives_sent;	/* ? */
	BUFFER			sendbuf;
	HDRCACHE		send_hdrcache;
	HDRCACHE		recv_hdrcache;
	HASHTABLE		send_pktcache;
	HASHTABLE		recv_pktcache;
	
	/* new diff stuff */
	BYTE			last_senddata[FURY_FRAME_MAX_DATA];
	int			last_sendsize;
	BYTE			last_recvdata[FURY_FRAME_MAX_DATA];
	int			last_recvsize;
	int			diff_reset;
	
	/* cache ptrs */
	FURY_CACHE_PTRS		cache_ptrs;
	
	/* if their buffer has been queued */
	int			buffer_queued;
	struct timeval		buffer_wait;
		
	/* options associated with this IP address */
	int			options;
	int			v_major, v_minor, v_build;
	
	/* if they don't want to be talked to */
	int			removed;
	
	/* amount of data we have sent */
	int			total_sent;
	int			total_recv;
	
} IPADDR_ENTRY;

typedef struct stHWIP_ENTRY {
	BYTE		hwaddr[6];		/* the hardware address */
	IPADDR_ENTRY	*ip;			/* pointer to IP address */
	struct timeval	last_ip_change_sent;	/* when we sent an IP change last */
	struct timeval	last_ip_req_sent;	/* when we last sent an IP req */
	int		ip_req_count;		/* how many requests we have sent out */
} HWIP_ENTRY;

typedef struct stIP_CHANGE {
	struct in_addr		addr;		/* IP addr */
	unsigned short		port;		/* the port */
	BYTE			hwaddr[6];	/* ethernet addr */
} IP_CHANGE;

/* protos */
int		get_ipaddr(char *str, struct in_addr *addr);

int		hwip_hash(HWIP_ENTRY *entry);
int		hwip_compare(HWIP_ENTRY *e1, HWIP_ENTRY *e2);
int		ipaddr_hash(IPADDR_ENTRY *entry);
int		ipaddr_compare(IPADDR_ENTRY *e1, IPADDR_ENTRY *e2);

#endif
