/*
** events.c -	event type functions for maintaining event lists
**
** Keith Zaback (Krz@wcug.wwu.edu) Apr 98
*/

#include "ipxfury.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>


extern GLOBALS	ifg;

/*
**	EVENT_CLEAN
*/
void event_clean(EVENTLIST *elist)
{
	/* easy stuff */
	free_eventlist(elist->next);
	elist->next = NULL;
	
	/* done */
	return;
}



/*
**	DO_EVENTS - does all the events 
*/
int do_events(EVENTLIST *elist, struct timeval *now)
{
	EVENTLIST	*current, *ptr, *todo;
	int		i;
	INTERFACE	*iface;
	
	/* remove waring */
	ptr = NULL;
	
	/* valid */
	if( elist == NULL ) {
		errormsg(0, "do_events", "NULL event list");
		return(0);
	}
	
	/* setup the ptr */
	current = elist->next;		/* the 1st item */
	todo	= NULL;
	
	/* loop thru the events to find ones todo */
	while( current != NULL ) {
		if( current->alarm.tv_sec <= now->tv_sec ) {
#if 0
			notice("Adding event type %d on the TODO list\n", current->type);
#endif
			if( todo == NULL ) {
				todo		= current;
				current		= current->next;
				todo->next	= NULL;
				ptr		= todo;
			} else {
				ptr->next  	= current;
				current		= current->next;
				ptr        	= ptr->next;
				ptr->next	= NULL;
			}
		} else {
			break;	/* jump out */
		}
	} 		
	
	/* patch the list */
	elist->next = current;
#if 0
	notice("New Event List:\n");
	event_dump(elist);
#endif

	/* process events on the todo list */
	while( todo != NULL ) {	
#if 0
		notice("Processing event type %d\n", todo->type);
#endif
		/* process */
		switch( todo->type ) {
			case ETYPE_CACHE_MAINT:
				iface = ifg.interfaces;
				for( i = 0; i < ifg.total_interfaces; i++ ) {
					if( iface->cache_ptrs.send_hdrcache != NULL ) {
						if( iface->type != FURY_IF_TYPE_UDP ) {
							hdrcache_maint(iface->cache_ptrs.send_hdrcache);
						} else {
							hash_enum_entries(&iface->data.u.u_ipaddrs, (void (*)(int, void *))udp_hdrcache_maint);
						}
					}
					iface++;
				}
				break;
			case ETYPE_FLCP_TIMEOUT:
				((void (*)(void *))todo->handler)(todo->user);
				break;
			case ETYPE_LCP_IGNORE:
				/* remove ignore flag */
				((INTERFACE *)todo->user)->flags &= ~FURY_IF_LCP_IGNORE;
				break;
			case ETYPE_PERIODIC_STATS:
				/* show the info */
				show_recent_stats();
				break;
			case ETYPE_RESET_CHECK:
				/* did we get an ack yet? */
				if( ifg.status == FSTATUS_RESETTING ) {
					/* nope, send another req */
					command_reset(1);
					/* add this event */
					add_event(ifg.events, ETYPE_RESET_CHECK, 2, 0, NULL, NULL);
					/* notice 'em */
					notice("[Event] Sending another reset request...\n");
				}
				break;
			case ETYPE_UDP_MAINT:
				/* do it */
				iface = interface_find_name("udp");
				if( NULL != iface ) {
					udp_maint(iface);
				}
				break;
				
			default:
				errormsg(0, "do_events", "unknown event type %d", todo->type);
				break;
		}
#if 0
		notice("Done with event type %d\n", todo->type);
#endif
		/* does this event respawn? */
		if( todo->respawn != 0 ) {
			/* adjust it's time */
			todo->alarm.tv_sec = now->tv_sec + todo->respawn;
			/* add it */
			ptr = todo->next;
			add_respawn(elist, todo);
			/* move along */
			todo = ptr;
		} else {
			/* free this node */
			ptr  = todo;
			todo = todo->next;
			free_event_node(ptr);
		} 
	}
	
	/* all done */
	return(1);	

}




/*
**	EVENT_DUMP - prints the events
*/
void event_dump(EVENTLIST *elist)
{
	int		minutes, seconds;
	struct timeval	now;
	
	/* valid list */
	if( elist == NULL ) {
		errormsg(0, "event_dump", "NULL event list");
		return;
	}
	
	/* get the now */
	gettimeofday(&now, NULL);
	
	/* move to the 1st */
	elist = elist->next;
	
	/* for each */
	while( elist != NULL ) {
		/* print the countdown time */
		seconds	= elist->alarm.tv_sec - now.tv_sec;
		minutes	= seconds / 60;
		seconds	= seconds % 60;
		notice("[%.2d:%.2d]", minutes, seconds);
		
		/* print the type */
		switch( elist->type ) {
			case ETYPE_CACHE_MAINT:
				notice("  Cache Maintence");
				break;
			case ETYPE_LCP_IGNORE:
				notice("  Ignore LCP     ");
				break;
			case ETYPE_FLCP_TIMEOUT:
				notice("  LCP Timeout    ");
				break;
			case ETYPE_PERIODIC_STATS:
				notice("  Device Stats   ");
				break;
			case ETYPE_RESET_CHECK:
				notice("  Reset Check    ");
				break;
			case ETYPE_UDP_MAINT:
				notice("  UDP Maintence  ");
				break;
			default:
				notice("  Event [%4d]   ", elist->type);
				break;
		}
		
		/* does it respawn? */
		if( elist->respawn != 0 ) {
			notice("   --  Respawn (%d seconds)\n", elist->respawn);
		} else {
			notice("   --  One-Shot\n");
		}
		
		/* move along */
		elist	= elist->next;
	}
	
	/* done */

}



/*
**	REMOVE_EVENTS - removes all events of a certain type
*/
int remove_events(EVENTLIST *elist, int type, void (*handler)(void *, int) )
{
	int		count;
	EVENTLIST	*ptr, *shadow;

	/* valid list */
	if( elist == NULL ) {
		errormsg(0, "remove_events", "NULL event list");
		return(0);
	}
	
	/* init */	
	count	= 0;
	shadow	= elist;
	ptr	= elist->next;	
	
	/* easy */
	while( ptr != NULL ) {
		if( (ptr->type == type) && (ptr->handler == handler) ) {
			shadow->next = ptr->next;
			free_event_node(ptr);
			ptr = shadow->next;
			count++;
		} else { 
			/* move along */
			shadow	= ptr;
			ptr	= ptr->next;
		}
	}
	
	/* done */
	return(count);
}

			



/*
**	ADD_RESPAWN - adds a respawner
*/
void add_respawn(EVENTLIST *elist, EVENTLIST *spawn)
{
	EVENTLIST	*ptr, *shadow;

#if 0
	notice("Adding respawn type %d\n", spawn->type);
#endif
	
	/* figure out where to put it */	
	shadow  = elist;		/* start at the head */
	ptr	= elist->next;		/* 1st entry */	
	
	while( ptr != NULL ) {
		/* does it happen before this one */
		if( spawn->alarm.tv_sec < ptr->alarm.tv_sec ) {
			break;
		}
		/* move along */
		shadow	= ptr;
		ptr	= ptr->next;
	}
	
	/* add it here */
	spawn->next = shadow->next;
	shadow->next = spawn;

#if 0
	notice("List after adding respawn:\n");
	event_dump(elist);
#endif
	
	/* done */
	return;

}



/*
**	ADD_EVENT - adds an event to the list
*/
int add_event(EVENTLIST *elist, int type, int seconds, int respawn, void (*handler)(void *, int), void *user)
{
	EVENTLIST	*newevent;
	struct timeval	now;

#if 0
	notice("add_event(else, %d, %d, %d, handler, user)\n", type, seconds, respawn);
#endif
	
	/* valid */
	if( elist == NULL ) {
		errormsg(0, "add_event", "NULL event list");
		return(0);
	}
	
	/* get now */
	gettimeofday(&now, NULL);
		
	/* get the ptr */
	newevent = get_event_node();
	if( newevent == NULL ) {
		errormsg(0, "add_event", "could not allocate a new node");
		return(0);
	}
	
	/* fill in the entry */
	newevent->type		= type;
	newevent->alarm.tv_sec	= now.tv_sec + seconds;
	newevent->respawn	= respawn;
	newevent->handler	= handler;
	newevent->user		= user;
	
	/* use add respawn */
	add_respawn(elist, newevent);

#if 0
	notice("List after adding:\n");
	event_dump(elist);
#endif
	
	/* done */
	return(1);

}



/*
**	GET_EVENT_NODE - returns a node
*/
EVENTLIST *get_event_node()
{	
	EVENTLIST	*temp;
	
	temp = (EVENTLIST *)malloc(sizeof(EVENTLIST));
	if( temp != NULL ) {
		memset(temp, 0, sizeof(EVENTLIST));
	}
	
	return(temp);
}



/*
**	FREE_EVENT_NODE - frees the memory
*/
void free_event_node(EVENTLIST *node)
{
	if( node != NULL ) {	
		free(node);
	}
}




/*
**	CREATE_EVENTLIST -
*/
EVENTLIST *create_eventlist()
{
	EVENTLIST	*temp;
	
	/* simple enough, just create a head node and return it */
	temp = (EVENTLIST *)malloc(sizeof(EVENTLIST));
	if( temp != NULL ) {
		/* clean it */
		memset(temp, 0, sizeof(EVENTLIST));
		
		/* setup some starting values */
		temp->type = ETYPE_LIST_HEAD;
		
		/* done */
		return(temp);
	}
	
	/* failed */
	return(NULL);

}


/*
**	FREE_EVENTLIST - frees the memory
*/
void free_eventlist(EVENTLIST *list)
{
	EVENTLIST	*shadow;
	
	/* here we go */
	while( list != NULL ) {
		shadow = list;
		list = list->next;
		free(shadow);
	}
	
	/* done */
		
}


