/*
** ipaddr.c - deals with IP address
**
** Keith Zaback (krz@wcug.wwu.edu) Sep 98
*/

#include "ipxfury.h"
#include <string.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>

extern GLOBALS	ifg;

/*
**	HWIP_HASH - hash function for hardware to ip table
*/
int hwip_hash(HWIP_ENTRY *entry)
{
	int		i;
	int		val;
	
	/* init */
	val = 0;
	
	/* do it */
	for( i = 0; i < 6; i++ ) {
		val ^= entry->hwaddr[i];
	}
	
	/* give it back */
	return(val);
}


/*
**	HWIP_COMPARE - 
*/
int hwip_compare(HWIP_ENTRY *e1, HWIP_ENTRY *e2)
{
	/* compare hwaddrs */
	return( memcmp(e1->hwaddr, e2->hwaddr, 6) );
}



/*
**	GET_IPADDR - converts name -> num
*/
int get_ipaddr(char *str, struct in_addr *addr)
{
	struct hostent		*hp;
	
	/* is it num & dots */
	if( inet_aton(str, addr) == 0 ) {
		/* try gethostbyname */
		ifg.alarm = 0;
		alarm(FURY_HOST_LOOKUP_WAIT);
		hp = gethostbyname(str);
		alarm(0);
		
		/* are we ok */
		if( ifg.alarm == 0x00 ) {
			if( NULL != hp ) {
				/* copy the addr */
				memcpy(addr, hp->h_addr, hp->h_length);
				return(1);
			}
		}
		
		/* did bad */
		return(0);
	}
	
	/* did ok */
	return(1);
}


/*
**	IPADDR_HASH - hash function
*/
int ipaddr_hash(IPADDR_ENTRY *entry)
{
	int		hash;
	unsigned char	*val;
	int		i;
	
	/* start */
	hash = 0x00;
	
	/* place ptr */
	val = (unsigned char *)&entry->addr;
	
	for( i = 0; i < sizeof(struct in_addr); i++ ) {
		hash += *val;
	}
	
	/* give it back */
	return(hash);
}


/*
**	IPADDR_COMPARE - compares them
*/
int ipaddr_compare(IPADDR_ENTRY *e1, IPADDR_ENTRY *e2)
{
	if( 0 == memcmp(&e1->addr, &e2->addr, sizeof(struct in_addr)) ) {
		/* same port? */
		if( e1->port == e2->port ) {
			return(0);
		}
	}
	
	/* nope */
	return(1);		
}
