/*
** misc.c - holds anything that doesn't really fit 
**
** Keith Zaback (krz@wcug.wwu.edu) Mar 98
*/

#include "ipxfury.h"
#include <stdio.h>
#include <stdarg.h>
#include <netinet/in.h> 

extern GLOBALS	ifg;


/*
**	PRINT_DATATRANS - makes numbers cool!
**
**	format:  111.22[BKMG]
**
*/
char *print_datatrans(int amt)
{
	static char	buffer[64];
	float		val;
	char		size;
	
	if( amt < 1000 ) {
		/* special case */
		sprintf(buffer, "%dB", amt);
		return(buffer);
	} else if( amt < 1000000 ) { 
		val = (float)amt / (float)1024;
		size = 'K';
	} else if( amt < 1000000000 ) {
		val = (float)amt / (float)(1024 * 1024);
		size = 'M';
	} else {
		val = (float)amt / (float)(1024 * 1024 * 1024);
		size = 'G';
	}

	/* make number */
	sprintf(buffer, "%.2f%c", val, size);
	
	/* here we go */
	return(buffer);

}


/*
**	PRINT_TIME
*/
void print_time(struct tm *t)
{
	char		*bonus;
	int		hour;
	
	if( t->tm_hour >= 12 ) {
		bonus	= "pm";
		hour = t->tm_hour - 12;
	} else {
		bonus 	= "am";
		hour = t->tm_hour;
	}

	/* last check */
	if( hour == 0 )
		hour = 12;
			
	/* print */
	notice("%2d:%.2d:%.2d%s", hour, t->tm_min, t->tm_sec, bonus);

	/* done */
	return;

}



/*
**	PRINT_IPX_ADDR
*/
void print_ipx_addr(IPXADDR *addr, int flags)
{
	BYTE		*ptr;
	int		i;
	
	/* print the network? */
	if( flags & PRINT_IPX_NETWORK ) {
		ptr = (BYTE *)&addr->network;
		for( i = 0; i < 4; i++ ) {
			notice("%.2x", *ptr);
			ptr++;
		}
		/* are we printing the host? */
		if( flags & PRINT_IPX_HOST )
			notice(":");
	}
		
	/* check for host */
	if( flags & PRINT_IPX_HOST ) {
		ptr = (BYTE *)&addr->host;
		for(i = 0; i < 6; i++ ) {
			notice("%.2x", *ptr);
			ptr++;
		}
		/* are we printing a port? */
		if( flags & PRINT_IPX_PORT )
			notice(":");
	}
	
	/* print the port? */
	if( flags & PRINT_IPX_PORT ) {
		ptr = (BYTE *)&addr->port;
		for( i = 0; i < 2; i++ ) {
			notice("%.2x", *ptr);
			ptr++;
		}
	}
	/* done */

}


/*
**	PRINT_IPX_FRAME - the frames are in network byte order, so convert
**			  the values to host, but _NOT_ in the data, we
**			  should not change the memory in any way!!!
*/
void print_ipx_frame(BYTE *ipxframe, int show_data)
{
	IPXHEADER	*ipxheader;
	int		i;
	BYTE		*ptr;
	int		length, srcport, destport, checksum;
	
	/* set the pointer so we can use it */
	ipxheader = (IPXHEADER *)ipxframe;
	
	/* do some conversions */
	length		= ntohs(ipxheader->length);
	checksum	= ntohs(ipxheader->checksum);
	srcport		= ntohs(ipxheader->ipx_src.port);
	destport	= ntohs(ipxheader->ipx_dest.port);
	
	/* print the source and destination */
	fprintf(ifg.logfile, "Src: ");
	ptr = (BYTE *)&ipxheader->ipx_src.host;
	for(i = 0; i < 6; i++, ptr++ )
		fprintf(ifg.logfile, "%.2X", *ptr);		
	fprintf(ifg.logfile, ":%.4X      Dest: ", srcport);
	ptr = (BYTE *)&ipxheader->ipx_dest.host;
	for( i = 0; i < 6; i++, ptr++ )
		fprintf(ifg.logfile, "%.2X", *ptr);
	fprintf(ifg.logfile, ":%.4X\n", destport);
	
	/* checksum, length, transport control */
/****************
	printf("Checksum: 0x%.4X   Length: 0x%.4X (%d)   TC: 0x%.2X\n",
	        checksum, length, length, ipxheader->control);
*******************/	  
	        
	/* do they want they data? */
	if( show_data ) {
		hexdump((ipxframe + sizeof(IPXHEADER)), (length - sizeof(IPXHEADER)));
	}
	
	/* done */
}



/*
**	PRINT_ETHER_ADDR
*/
char *print_ether_addr(unsigned char *addr)
{
	int		i;
	static char	*ptr, buffer[24];
	
	ptr = buffer;
	
	/* print the addr */
	for( i = 0; i < 5; i++ ) {
		ptr += sprintf(ptr, "%.2x:", *addr);
		addr++;
	}
	ptr += sprintf(ptr, "%.2x", *addr);
	
	/* null end */
	*ptr = 0x00;
	
	/* give it back */
	return(buffer);

}



